#!/usr/bin/perl 

################################################################################
# Name:        adminDB.pl
# Version:     1.1
# Author:      Reinhard E. Voglmaier
#              GlaxoWellcome
#              rv33100@GlaxoWellcome.co.uk
# Date:        12.04.2000
# Description: Library Routines for Input/Output/Dump of databases
# Copyright:   GPL
################################################################################

# Configuration including Publish.cfg
# We get the place where the database lives in


use AnyDBM_File; 
use POSIX(O_CREAT,O_RDWR,O_RDONLY); 
require "Publish.cfg" ;

my %AccessList ;
my %Inodes ;
my %Users ;

#
#-------------- create/open/close the databases -------------------
#               public Interface
sub create_databases {
    my ( $writable ) = @_ ;
    my ( $Permissions ) = $writable ? O_CREAT : O_RDONLY;
    # We have to change the ownership in order to let access the DB via Web
    if ( _tie_DB($Permissions)) {
        opendir(DBD,$DB) || die "Could not initialize DB in $DB " ;
        for $File ( grep(!/^\.\.?$/,readdir(DBD)) ) {
             my $DBFile=$DB.$File ;
             chown ((getpwnam($DB_Owner))[2],(getpwnam($DB_Owner_Group))[3],$DBFile) ;
             chmod(0744,$DBFile) ;
        }
        close(DBD);
        chmod(0744,$DB) ;
        return ( 1 ) ;
    }
}
    
sub open_databases {
    my ( $writable ) = @_ ;
    my ( $Permissions ) = $writable ? O_RDWR : O_RDONLY;
    return ( _tie_DB($Permissions)) ;
}
    

sub close_databases {
    untie(%AccessList) ;
    untie(%Inodes) ;
    untie(%Users) ;
}

#               private Functions
sub _tie_DB {
    my ( $Permissions ) = @_ ;

    tie(%AccessList, AnyDBM_File,$DB."AccessList",    $Permissions,0644) || die "tie: $!\n" ;
    tie(%Inodes,     AnyDBM_File,$DB."Inodes",        $Permissions,0644) || die "tie: $!\n" ;
    tie(%Users,      AnyDBM_File,$DB."Users",         $Permissions,0644) || die "tie: $!\n" ;
    1;
}

#
#-------------------- dump the databases  -----------------------
#
sub dump_user_list {
    foreach ( keys %AccessList) {
       printf("%s\t%s:\n",$_,getUserName($_));
       @List = split(';;',$AccessList{$_}) ;
       foreach $Element ( @List ) {
            printf("\t\t%s\n",$Element);
       }
       print "\n";
    }
}

sub dump_databases {
    print "Dump of AccessList:\n" ;
    foreach ( keys %AccessList) {
       print "$_     \t $AccessList{$_} \n" ;
    }
    print "Dump of Inodes:\n" ;
    foreach ( keys %Inodes) {
       print "$_     \t $Inodes{$_} \n" ;
    }
    print "Dump of Users:\n" ;
    foreach ( keys %Users ) {
       print "$_     \t $Users{$_} \n" ;
    }
}

#-------------------- reset the databases -----------------------
sub reset_databases {
    foreach ( keys %AccessList) {
       delete $AccessList{$_};
    }
    foreach ( keys %Inodes) {
       delete $Inodes{$_};
    }
    foreach ( keys %Users ) {
       delete $Users{$_};
    }
}

sub cleanUsers {
    foreach ( keys %Users) {
       if ( $_ eq '' ) {
          delete $Users{$_};
          print "vuoto , cleaned . . .\n" ;
       } 
    }
}

#-------------------- add an entry -----------------------
# List and Dir is substantially the same inasmuch it accesses the same data structure
# the difference is the underlying concept: Dir is an area of files and directories 
#                                  meanwhile List is a Directory containing files ( and 
#                                  only files, no dirs ) to be put in production
# Currently NOT USED !!!
sub addList {
    my ($DirName) = @_ ;
    my $Type = "List" ;
    my $Modified = "Never" ;
    my $UserId = "nobody" ;
    my $DirProperties = join(';;',$Type,$Modified,$UserId) ;
    $DirName = "/".$DirName if ( $DirName !~ /^\// ) ;
    $Inodes{$DirName} = $DirProperties ;
    return 1 ;
}
sub addDir {
    my ($DirName) = @_ ;
    my $Type = "Area" ;
    my $Modified = "Never" ;
    my $UserId = "nobody" ;
    my $DirProperties = join(';;',$Type,$Modified,$UserId) ;
    $DirName = "/".$DirName if ( $DirName !~ /^\// ) ;
    $Inodes{$DirName} = $DirProperties ;
    return 1 ;
}

sub addUser {
    my ($UserId,$UserName) = @_ ;
    if ( !$Users{$UserId} ) {
       $Users{$UserId} = $UserName ;
    }
    return 1;
}
sub addAdmin {
    my ( $UserId, $User ) = @_ ;
    my $UserName ;
    my $Role ;

    if ( $Users{$UserId} ) {
       ($User,$Role) = split(';;',$Users{$UserId}) ;
    }
    $Role = "Admin" ;
    $User =~ s/\n// ;
    $UserName = join(';;',$User,$Role) ;
    $Users{$UserId} = $UserName ;
}

sub addPermission {
    my ($UserId,$Directory) = @_ ;
    
    if ( $DirString = $AccessList{$UserId} ) {          # is there any entry ?
        my @DirList = split(';;',$DirString) ;          # yes.
        my $Dir ;
        foreach $Dir ( @DirList ) {
           return if ( $Dir eq $Directory ) ;           # The entry does exist. Nothing to do .
        }
        $DirList[@DirList] = $Directory ;
        @tmp = sort ( @DirList ) ;
        $AccessList{$UserId} = join ( ';;',@tmp ) ;
    } else {                                            # no, it's the first
        $AccessList{$UserId} = $Directory ;
    }
}


#-------------------- modify an entry -----------------------

sub modifyDir {
    my ($DirName,$UserId,$Modified) = @_ ;
    my $Type = "Area" ;
    my $DirProperties = join(';;',$Type,$Modified,$UserId) ;
    $Inodes{$DirName} = $DirProperties ;
}
sub modifyList {
    my ($DirName,$UserId,$Modified) = @_ ;
    my $Type = "List" ;
    my $DirProperties = join(';;',$Type,$Modified,$UserId) ;
    $Inodes{$DirName} = $DirProperties ;
}

#-------------------- delete an entry -----------------------

sub delUser {
    my ( $UserId ) = @_ ;
    delete $Users{$UserId} ;
    return 1;
}

sub delPermission {
    my ( $UserId ) = @_ ;
    delete $AccessList{$UserId};
}
sub delInode {
    my ($DirName) = @_ ;
	if ( $Inodes{$DirName} ) {
    	delete $Inodes{$DirName} ;
		return(1);
	} else {
		return (-1) ;
	}
}

#-------------------- get an entry -----------------------

#-------------------- get an entry -----------------------

sub getDirKeys {
    return ( keys %Inodes) ;
}

sub getDirEntry {
    my ( $Key ) = @_ ;
    my $DirEntry = $Inodes{$Key} ;
    return split(';;',$DirEntry) ;
}

sub getAccessKeys {
    return ( keys %AccessList );
}

sub getUserKeys {
    return ( keys %Users ) ;
}

sub getAccessEntry {
    my ( $Key ) = @_ ;
    my $AccessEntry = $AccessList{$Key} ;
    return split(';;',$AccessEntry);
}

sub getUserInfo {
    my ( $Key ) = @_ ;
    if ( $Users{$Key} ) {
        return split(';;',$Users{$Key}) ;
    }
}

sub getUserName {
    my ( $Key ) = @_ ;
    my $User ;
    my $Role ;

    if ( $Users{$Key} ) {
        ($User,$Role) = split(';;',$Users{$Key}) ;
    } else {
        ($User,$Role) = ('','') ;
    }
    return $User ;
}

sub ExitWithError {
    my ( $Error ) = @_ ;

    printf("%s\n",$Error);
    exit(-99);
}

1;
